@@ -5,7 +5,7 @@ use chrono::{
     Datelike,
     NaiveDateTime as ChronoDateTime,
     Duration as ChronoDuration,
-    Weekday as ChronoWeekday, Timelike
+    Weekday as ChronoWeekday
 };
 
 use crate::lexer::Lexeme;
@@ -27,151 +27,103 @@ pub enum DateTime {
 
 impl DateTime {
     /// Parse a datetime from a slice of lexemes
-    pub fn parse(l: &[Lexeme]) -> Result<Option<(Self, usize)>, String> {
+    pub fn parse(l: &[Lexeme]) -> Option<(Self, usize)> {
         let mut tokens = 0;
-
-        if l.get(tokens) == Some(&Lexeme::A) || l.get(tokens) == Some(&Lexeme::The) || l.get(tokens) == Some(&Lexeme::An) {
+        if l.get(tokens) == Some(&Lexeme::Now) {
             tokens += 1;
+            return Some((Self::Now, tokens))
+        }
 
-            if let Some((unit, t)) = Unit::parse(&l[tokens..]) {
-                tokens += t;
-                if Some(&Lexeme::After) == l.get(tokens) || Some(&Lexeme::From) == l.get(tokens) {
-                    tokens += 1;
-                    let datetime = DateTime::parse(&l[tokens..])?;
-
-                    if datetime.is_none() {
-                        return Err("Expected datetime".into());
-                    }
-
-                    let (datetime, t) = datetime.unwrap();
-                    tokens += t;
-
-                    Ok(Some((Self::After(Duration {unit, num: 1}, Box::new(datetime)), tokens)))
-                } else if Some(&Lexeme::Before) == l.get(tokens) {
-                    tokens += 1;
-                    let datetime = DateTime::parse(&l[tokens..])?;
-
-                    if datetime.is_none() {
-                        return Err("Expected datetime".into());
-                    }
-
-                    let (datetime, t) = datetime.unwrap();
-                    tokens += t;
-
-                    Ok(Some((Self::Before(Duration { unit, num: 1 }, Box::new(datetime)), tokens)))
-                } else if Some(&Lexeme::Ago) == l.get(tokens) {
-                    tokens += 1;
-                    Ok(Some((Self::Ago(Duration{unit, num: 1}), tokens)))
-
-                } else {
-                    Err("Expected 'after' 'before' or 'ago'".into())
-                }
-            } else {
-                Err("Expected Unit".into())
-            }
-
-        } else if l.get(tokens) == Some(&Lexeme::Now) {
-            tokens += 1;
-            return Ok(Some((Self::Now, tokens)))
-        } else if let Ok(Some((dur, t))) = Duration::parse(&l[tokens..]) {
+        tokens = 0;
+        if let Some((dur, t)) = Duration::parse(&l[tokens..]) {
             tokens += t;
 
-            if Some(&Lexeme::After) == l.get(tokens) || Some(&Lexeme::From) == l.get(tokens) {
+            if Some(&Lexeme::After) == l.get(tokens)
+            || Some(&Lexeme::From) == l.get(tokens) {
                 tokens += 1;
 
-                let datetime = DateTime::parse(&l[tokens..])?;
-
-                if datetime.is_none() {
-                    return Err("Expected datetime".into());
+                if let Some((datetime, t)) = DateTime::parse(&l[tokens..]) {
+                    tokens += t;
+                    return Some((Self::After(dur, Box::new(datetime)), tokens));
                 }
-
-                let (datetime, t) = datetime.unwrap();
-                tokens += t;
-
-                Ok(Some((Self::After(dur, Box::new(datetime)), tokens)))
             } else if Some(&Lexeme::Before) == l.get(tokens) {
                 tokens += 1;
 
-                let datetime = DateTime::parse(&l[tokens..])?;
-
-                if datetime.is_none() {
-                    return Err("Expected datetime".into());
+                if let Some((datetime, t)) = DateTime::parse(&l[tokens..]) {
+                    tokens += t;
+                    return Some((Self::Before(dur, Box::new(datetime)), tokens));
                 }
-
-                let (datetime, t) = datetime.unwrap();
-                tokens += t;
-
-                Ok(Some((Self::Before(dur, Box::new(datetime)), tokens)))
             } else if Some(&Lexeme::Ago) == l.get(tokens) {
                 tokens += 1;
-                Ok(Some((Self::Ago(dur), tokens)))
-            } else {
-                Err("Expected 'after' or 'before'".into())
+                return Some((Self::Ago(dur), tokens));
             }
-        } else if let Some((date, t)) = Date::parse(&l[tokens..])? {
+        }
+
+        tokens = 0;
+        if let Some((date, t)) = Date::parse(&l[tokens..]) {
             tokens += t;
             if l.get(tokens) == Some(&Lexeme::Comma) {
                 tokens += 1;
             }
 
-            let (time, t) = Time::parse(&l[tokens..])?;
-            tokens += t;
-
-            Ok(Some((Self::DateTime(date, time), tokens)))
-        } else {
-            Ok(None)
+            if let Some((time, t)) = Time::parse(&l[tokens..]) {
+                tokens += t;
+                return Some((Self::DateTime(date, time), tokens));
+            }
         }
+
+        None
     }
 
     /// Convert a parsed DateTime to chrono's NaiveDateTime
-    pub fn to_chrono(&self) -> ChronoDateTime {
+    pub fn to_chrono(&self, default: ChronoTime) -> ChronoDateTime {
         match self {
             DateTime::Now => Local::now().naive_local(),
             DateTime::DateTime(date, time) => {
                 let date = date.to_chrono();
-                let time = time.to_chrono();
+                let time = time.to_chrono(default);
 
                 ChronoDateTime::new(date, time)
             }
             DateTime::After(dur, date) => {
-                let mut date = date.to_chrono();
+                let mut date = date.to_chrono(default);
 
                 if dur.convertable() {
                     date + dur.to_chrono()
                 } else {
-                    match dur.unit {
+                    match dur.unit() {
                         Unit::Month => {
                             if date.month() == 12 {
                                 date = date.with_month(1).unwrap();
                                 date.with_year(date.year() + 1).unwrap()
                             } else {
-                                date.with_month(date.month()+dur.num).unwrap()
+                                date.with_month(date.month()+dur.num()).unwrap()
                             }
                         }
                         Unit::Year => {
-                            date.with_year(date.year()+dur.num as i32).unwrap()
+                            date.with_year(date.year()+dur.num() as i32).unwrap()
                         }
                         _ => unreachable!()
                     }
                 }
             }
             DateTime::Before(dur, date) => {
-                let mut date = date.to_chrono();
+                let mut date = date.to_chrono(default);
 
                 if dur.convertable() {
                     date - dur.to_chrono()
                 } else {
-                    match dur.unit {
+                    match dur.unit() {
                         Unit::Month => {
                             if date.month() == 1 {
                                 date = date.with_month(12).unwrap();
                                 date.with_year(date.year() - 1 as i32).unwrap()
                             } else {
-                                date.with_month(date.month()-dur.num).unwrap()
+                                date.with_month(date.month()-dur.num()).unwrap()
                             }
                         }
                         Unit::Year => {
-                            date.with_year(date.year()-dur.num as i32).unwrap()
+                            date.with_year(date.year()-dur.num() as i32).unwrap()
                         }
                         _ => unreachable!()
                     }
@@ -183,17 +135,17 @@ impl DateTime {
                 if dur.convertable() {
                     date - dur.to_chrono()
                 } else {
-                    match dur.unit {
+                    match dur.unit() {
                         Unit::Month => {
                             if date.month() == 1 {
                                 date = date.with_month(12).unwrap();
                                 date.with_year(date.year() - 1 as i32).unwrap()
                             } else {
-                                date.with_month(date.month()-dur.num).unwrap()
+                                date.with_month(date.month()-dur.num()).unwrap()
                             }
                         }
                         Unit::Year => {
-                            date.with_year(date.year()-dur.num as i32).unwrap()
+                            date.with_year(date.year()-dur.num() as i32).unwrap()
                         }
                         _ => unreachable!()
                     }
@@ -218,77 +170,82 @@ pub enum Date {
 }
 
 impl Date {
-    fn parse(l: &[Lexeme]) -> Result<Option<(Self, usize)>, String> {
+    fn parse(l: &[Lexeme]) -> Option<(Self, usize)> {
         let mut tokens = 0;
 
         if let Some(&Lexeme::Today) = l.get(tokens) {
             tokens += 1;
-            Ok(Some((Self::Today, tokens)))
-        } else if let Some(&Lexeme::Tomorrow) = l.get(tokens) {
-            tokens += 1;
-            Ok(Some((Self::Tomorrow, tokens)))
-        } else if let Some(&Lexeme::Yesterday) = l.get(tokens) {
-            tokens += 1;
-            Ok(Some((Self::Yesterday, tokens)))
-        } else if let Some((month, t)) = Month::parse(&l[tokens..]) {
-            tokens += t;
-            if let Some((day, t)) = Num::parse(&l[tokens..])? {
-                tokens += t;
-                if l.get(tokens) == Some(&Lexeme::Comma) {
-                    tokens += 1;
-                }
+            return Some((Self::Today, tokens));
+        }
 
-                if let Some((year, t)) = Num::parse(&l[tokens..])? {
-                    tokens += t;
-                    Ok(Some((Self::MonthDayYear(month, day, year), tokens)))
-                } else {
-                    Ok(Some((Self::MonthDay(month, day), tokens)))
-                }
-            } else {
-                Err("Expected day".into())
-            }
-        } else if let Some((relspec, t)) = RelativeSpecifier::parse(&l[tokens..]) {
+        tokens = 0;
+        if let Some(&Lexeme::Tomorrow) = l.get(tokens) {
+            tokens += 1;
+            return Some((Self::Tomorrow, tokens));
+        }
+
+        tokens = 0;
+        if let Some(&Lexeme::Yesterday) = l.get(tokens) {
+            tokens += 1;
+            return Some((Self::Yesterday, tokens));
+        }
+
+        tokens = 0;
+        if let Some((month, t)) = Month::parse(&l[tokens..]) {
             tokens += t;
+
+            let (day, t) = Num::parse(&l[tokens..])?;
+            tokens += t;
+
+            if l.get(tokens) == Some(&Lexeme::Comma) {
+                tokens += 1;
+            }
+
+            if let Some((year, t)) = Num::parse(&l[tokens..]) {
+                tokens += t;
+                return Some((Self::MonthDayYear(month, day, year), tokens));
+            } else {
+                return Some((Self::MonthDay(month, day), tokens));
+            }
+        }
+
+        tokens = 0;
+        if let Some((relspec, t)) = RelativeSpecifier::parse(&l[tokens..]) {
+            tokens += t;
+
             if let Some((weekday, t)) = Weekday::parse(&l[tokens..]) {
                 tokens += t;
-                Ok(Some((Self::Relative(relspec, weekday), tokens)))
-            } else {
-                Err("Expected weekday".into())
+                return Some((Self::Relative(relspec, weekday), tokens));
             }
         } else if let Some((weekday,t)) = Weekday::parse(&l[tokens..]) {
             tokens += t;
-            Ok(Some((Self::Weekday(weekday), tokens)))
-        } else if let Some((month, t)) = Num::parse(&l[tokens..])? {
+            return Some((Self::Weekday(weekday), tokens));
+        } else if let Some((month, t)) = Num::parse(&l[tokens..]) {
             tokens += t;
-            if l.get(tokens) != Some(&Lexeme::Slash)
-            && l.get(tokens) != Some(&Lexeme::Dash)
-            {
-                return Err("expected / or -".into());
-            }
-
-            tokens += 1;
-
-            if let Some((day, t)) = Num::parse(&l[tokens..])? {
-                tokens += t;
-                if l.get(tokens) == Some(&Lexeme::Slash)
-                || l.get(tokens) == Some(&Lexeme::Dash)
-                {
+            if let Some(delim) = l.get(tokens) {
+                if delim == &Lexeme::Slash || delim == &Lexeme::Dash {
+                    // Consume slash or dash
                     tokens += 1;
-                    if let Some((year, t)) = Num::parse(&l[tokens..])? {
+
+                    if let Some((day, t)) = Num::parse(&l[tokens..]) {
                         tokens += t;
-                        Ok(Some((Self::MonthNumDayYear(month, day, year), tokens)))
-                    } else {
-                        Ok(Some((Self::MonthNumDay(month, day), tokens)))
+                        if l.get(tokens)? == delim {
+                            // Consume slash or dash
+                            tokens += 1;
+
+                            let (year, t) = Num::parse(&l[tokens..])?;
+                            tokens += t;
+                            return Some((Self::MonthNumDayYear(month, day, year), tokens));
+                        } else {
+                            return Some((Self::MonthNumDay(month, day), tokens));
+                        }
+
                     }
-                } else {
-                    Ok(Some((Self::MonthNumDay(month, day), tokens)))
                 }
-            } else {
-                Err("Expected day".into())
             }
-        } else {
-            Ok(None)
         }
+
+        None
     }
 
     fn to_chrono(&self) -> ChronoDate {
@@ -303,7 +260,7 @@ impl Date {
                 today + ChronoDuration::days(1)
             }
             Date::MonthNumDay(month, day) => {
-                let today = Local::now().naive_local().date();
+                let today = Local::now().naive_local();
                 ChronoDate::from_ymd(today.year(), *month, *day)
             }
             Date::MonthNumDayYear(month, day, year) => {
@@ -321,7 +278,7 @@ impl Date {
                 ChronoDate::from_ymd(year as i32, *month, *day)
             }
             Date::MonthDay(month, day) => {
-                let today = Local::now().naive_local().date();
+                let today = Local::now().naive_local();
                 let month = *month as u32;
                 ChronoDate::from_ymd(today.year(), month, *day)
             }
@@ -467,39 +424,36 @@ pub enum Time {
 }
 
 impl Time {
-    fn parse(l: &[Lexeme]) -> Result<(Self, usize), String> {
+    fn parse(l: &[Lexeme]) -> Option<(Self, usize)> {
         let mut tokens = 0;
 
-        if let Some((hour, t)) = Num::parse(&l[tokens..])? {
+        if let Some((hour, t)) = Num::parse(&l[tokens..]) {
             tokens += t;
-            if l.get(tokens) != Some(&Lexeme::Colon) {
-                return Err("Expected Colon".into());
-            }
+            if l.get(tokens) == Some(&Lexeme::Colon) {
+                tokens += 1;
 
-            tokens += 1;
-
-            if let Some((min, t)) = Num::parse(&l[tokens..])? {
-                tokens += t;
-                if let Some(&Lexeme::AM) = l.get(tokens) {
-                    tokens += 1;
-                    Ok((Time::HourMinAM(hour, min), tokens))
-                } else if let Some(&Lexeme::PM) = l.get(tokens) {
-                    tokens += 1;
-                    Ok((Time::HourMinPM(hour, min), tokens))
-                } else {
-                    Ok((Time::HourMin(hour, min), tokens))
+                if let Some((min, t)) = Num::parse(&l[tokens..]) {
+                    tokens += t;
+                    if let Some(&Lexeme::AM) = l.get(tokens) {
+                        tokens += 1;
+                        return Some((Time::HourMinAM(hour, min), tokens));
+                    } else if let Some(&Lexeme::PM) = l.get(tokens) {
+                        tokens += 1;
+                        return Some((Time::HourMinPM(hour, min), tokens));
+                    } else {
+                        return Some((Time::HourMin(hour, min), tokens));
+                    }
                 }
-            } else {
-                Err("Expected minute".into())
             }
-        } else {
-            Ok((Self::Empty, tokens))
         }
+
+        tokens = 0;
+        Some((Self::Empty, tokens))
     }
 
-    fn to_chrono(&self) -> ChronoTime {
+    fn to_chrono(&self, default: ChronoTime) -> ChronoTime {
         match *self {
-            Time::Empty => Local::now().naive_local().time(),
+            Time::Empty => default,
             Time::HourMin(hour, min) => {
                 ChronoTime::from_hms(hour, min, 0)
             }
@@ -514,39 +468,83 @@ impl Time {
 }
 
 #[derive(Debug, Eq, PartialEq)]
-pub struct Duration {
-    num: u32,
-    unit: Unit
+pub enum Article {
+    A,
+    An,
+    The
+}
+
+impl Article {
+    fn parse(l: &[Lexeme]) -> Option<(Self, usize)> {
+        match l.get(0) {
+            Some(Lexeme::A) => Some((Self::A, 1)),
+            Some(Lexeme::An) => Some((Self::An, 1)),
+            Some(Lexeme::The) => Some((Self::The, 1)),
+            _ => None
+        }
+    }
+}
+
+
+#[derive(Debug, Eq, PartialEq)]
+pub enum Duration {
+    Article(Unit),
+    Specific(u32, Unit)
 }
 
 impl Duration {
-    fn parse(l: &[Lexeme]) -> Result<Option<(Self, usize)>, String> {
+    fn parse(l: &[Lexeme]) -> Option<(Self, usize)> {
         let mut tokens = 0;
 
-        if let Some((num, t)) = Num::parse(&l[tokens..])? {
+        if let Some((num, t)) = Num::parse(&l[tokens..]) {
             tokens += t;
             if let Some((u, t)) = Unit::parse(&l[tokens..]) {
                 tokens += t;
-                Ok(Some((Self {num, unit: u}, tokens)))
-            } else {
-                Err("Expected Unit while parsing Duration".into())
+                return Some((Self::Specific(num, u), tokens));
             }
-        } else {
-            Ok(None)
+        }
+
+        tokens = 0;
+        if let Some((_, t)) = Article::parse(l) {
+            tokens += t;
+            if let Some((u, t)) = Unit::parse(&l[tokens..]) {
+                tokens += t;
+                return Some((Self::Article(u), tokens));
+            }
+        }
+
+        None
+    }
+
+    fn unit(&self) -> &Unit {
+        match self {
+            Duration::Article(u) => u,
+            Duration::Specific(_, u) => u
+        }
+    }
+
+    fn num(&self) -> u32 {
+        match *self {
+            Duration::Article(_) => 1,
+            Duration::Specific(num, _) => num
         }
     }
 
     fn convertable(&self) -> bool {
-        self.unit != Unit::Month &&
-        self.unit != Unit::Year
+        let unit = self.unit();
+        unit != &Unit::Month &&
+        unit != &Unit::Year
     }
 
     fn to_chrono(&self) -> ChronoDuration {
-        match self.unit {
-            Unit::Day => ChronoDuration::days(self.num as i64),
-            Unit::Week => ChronoDuration::weeks(self.num as i64),
-            Unit::Hour => ChronoDuration::hours(self.num as i64),
-            Unit::Minute => ChronoDuration::minutes(self.num as i64),
+        let unit = self.unit();
+        let num = self.num();
+
+        match unit {
+            Unit::Day => ChronoDuration::days(num as i64),
+            Unit::Week => ChronoDuration::weeks(num as i64),
+            Unit::Hour => ChronoDuration::hours(num as i64),
+            Unit::Minute => ChronoDuration::minutes(num as i64),
             _ => unreachable!()
         }
     }
@@ -659,94 +657,98 @@ impl NumDouble {
         let mut tokens = 0;
 
         if let Some((tens, t)) = Tens::parse(&l[tokens..]) {
+            println!("tens: {tens:?}");
             tokens += t;
+
             if Some(&Lexeme::Dash) == l.get(tokens) {
                 tokens += 1;
             }
 
             let (ones, t) = Ones::parse(&l[tokens..]).unwrap_or((0, 0));
             tokens += t;
-            Some((tens + ones, tokens))
-        } else if let Some((teens, t)) = Teens::parse(&l[tokens..]) {
+            return Some((tens + ones, tokens))
+        }
+
+        tokens = 0;
+        if let Some((teens, t)) = Teens::parse(&l[tokens..]) {
             tokens += t;
-            Some((teens, tokens))
-        } else if let Some((ones, t)) = Ones::parse(&l[tokens..]) {
+            return Some((teens, tokens));
+        }
+
+        tokens = 0;
+        if let Some((ones, t)) = Ones::parse(&l[tokens..]) {
             tokens += t;
-            Some((ones, tokens))
-        } else {
-            if let Some(Lexeme::Num(n)) = l.get(tokens) {
-                tokens += 1;
-                if *n < 100 && *n > 19 {
-                    Some((*n, tokens))
-                } else {
-                    None
-                }
-            } else {
-                None
+            return Some((ones, tokens));
+        }
+
+        tokens = 0;
+        if let Some(Lexeme::Num(n)) = l.get(tokens) {
+            tokens += 1;
+            if *n < 100 && *n > 19 {
+                return Some((*n, tokens));
             }
         }
+
+        None
     }
 }
 
 struct NumTriple;
 impl NumTriple {
-    fn parse(l: &[Lexeme]) -> Result<Option<(u32, usize)>, String> {
+    fn parse(l: &[Lexeme]) -> Option<(u32, usize)> {
         let mut tokens = 0;
 
         if let Some((ones, t)) = Ones::parse(&l[tokens..]) {
             tokens += t;
 
-            if Some(&Lexeme::Hundred) != l.get(tokens) {
-                // Try parsing instead as num_double
-                tokens -= t;
-                if let Some((double, t)) = NumDouble::parse(&l[tokens..]) {
-                    return Ok(Some((double, t)));
-                } else {
-                    return Err("Expected 'hundred'".into());
+            if Some(&Lexeme::Hundred) == l.get(tokens) {
+                // Consume 'Hundred'
+                tokens += 1;
+
+                let required = Some(&Lexeme::And) == l.get(tokens);
+                if required { tokens += 1; }
+                let double = NumDouble::parse(&l[tokens..]);
+
+                if !required || double.is_some() {
+                    let (double, t) = double.unwrap_or((0, 0));
+                    tokens += t;
+
+                    return Some((ones*100+double, tokens));
                 }
             }
+        }
+
+        tokens = 0;
+        if Some(&Lexeme::Hundred) == l.get(tokens) {
             tokens += 1;
 
             let required = Some(&Lexeme::And) == l.get(tokens);
             if required { tokens += 1; }
             let double = NumDouble::parse(&l[tokens..]);
 
-            if required && double.is_none() {
-                return Err("Expected number after 'and'".into());
+            if !required || double.is_some() {
+                let (double, t) = double.unwrap_or((0, 0));
+                tokens += t;
+
+                return Some((100+double, tokens));
             }
+        }
 
-            let (double, t) = double.unwrap_or((0, 0));
+        tokens = 0;
+        if let Some((num_double, t)) = NumDouble::parse(&l[tokens..]) {
             tokens += t;
+            return Some((num_double, tokens));
+        }
 
-            Ok(Some((ones*100+double, tokens)))
-        } else if Some(&Lexeme::Hundred) == l.get(tokens) {
-            tokens += 1;
-
-            let required = Some(&Lexeme::And) == l.get(tokens);
-            if required { tokens += 1; }
-            let double = NumDouble::parse(&l[tokens..]);
-
-            if required && double.is_none() {
-                return Err("Expected number after 'and'".into());
-            }
-
-            let (double, t) = double.unwrap_or((0, 0));
-            tokens += t;
-
-            Ok(Some((100+double, tokens)))
-        } else if let Some((num_double, t)) = NumDouble::parse(&l[tokens..]) {
-            tokens += t;
-            Ok(Some((num_double, tokens)))
-        } else if let Some(&Lexeme::Num(n)) = l.get(tokens) {
+        tokens = 0;
+        if let Some(&Lexeme::Num(n)) = l.get(tokens) {
             tokens += 1;
             if n > 99 && n < 1000 {
-                Ok(Some((n, tokens)))
-            } else {
-                Ok(None)
+                return Some((n, tokens));
             }
-        } else {
-            Ok(None)
         }
+
+        None
     }
 }
 
@@ -764,57 +766,67 @@ impl NumTripleUnit {
 
 struct Num;
 impl Num {
-    fn parse(l: &[Lexeme]) -> Result<Option<(u32, usize)>, String> {
+    fn parse(l: &[Lexeme]) -> Option<(u32, usize)> {
+        println!("Parse NUM: {:?}", l.get(0));
         let mut tokens = 0;
 
-        if let Some((triple, t)) = NumTriple::parse(&l[tokens..])? {
+        // <num_triple>
+        if let Some((triple, t)) = NumTriple::parse(&l[tokens..]) {
+            println!("triple: {triple:?}");
             tokens += t;
-            let unit = NumTripleUnit::parse(&l[tokens..]);
 
-            if unit.is_none() {
-                return Ok(Some((triple, tokens)))
+            // <num_triple_unit>
+            if let Some((unit, t)) = NumTripleUnit::parse(&l[tokens..]) {
+                tokens += t;
+
+                let required = Some(&Lexeme::And) == l.get(tokens);
+                if required { tokens += 1; }    // Consume and
+                let num = Num::parse(&l[tokens..]);
+                println!("NUM: {num:?}");
+
+                if !required || num.is_some() {
+                    let (num, t) = num.unwrap_or((0, 0));
+                    tokens += t;
+
+                    return Some((triple*unit+num, tokens));
+                }
             }
+        }
 
-            let (unit, t) = unit.unwrap();
+        tokens = 0;
+        // <num_triple_unit>
+        if let Some((unit, t)) = NumTripleUnit::parse(&l[tokens..]) {
             tokens += t;
 
             let required = Some(&Lexeme::And) == l.get(tokens);
-            if required { tokens += 1; }
-            let num = Num::parse(&l[tokens..])?;
+            if required { tokens += 1; }    // Consume and
+            let num = Num::parse(&l[tokens..]);
 
-            if required && num.is_none() {
-                return Err("Expected num".into());
+            if num.is_some() || !required {
+                let (num, t) = num.unwrap_or((0, 0));
+                tokens += t;
+
+                return Some((unit + num, tokens));
             }
+        }
 
-            let (num, t) = num.unwrap_or((0, 0));
+        // <num_triple>
+        tokens = 0;
+        if let Some((num, t)) = NumTriple::parse(&l[tokens..]) {
             tokens += t;
+            return Some((num, tokens))
+        }
 
-            Ok(Some((triple*unit+num, tokens)))
-        } else if let Some((unit, t)) = NumTripleUnit::parse(&l[tokens..]) {
-            tokens += t;
-
-            let required = Some(&Lexeme::And) == l.get(tokens);
-            if required { tokens += 1; }
-            let num = Num::parse(&l[tokens..])?;
-
-            if required && num.is_none() {
-                return Err("Expected num".into());
-            }
-
-            let (num, t) = num.unwrap_or((0, 0));
-            tokens += t;
-
-            Ok(Some((unit + num, tokens)))
-        } else if let Some(&Lexeme::Num(n)) = l.get(tokens) {
+        tokens = 0;
+        // NUM
+        if let Some(&Lexeme::Num(n)) = l.get(tokens) {
             tokens += 1;
             if n >= 1000 {
-                Ok(Some((n, tokens)))
-            } else {
-                Ok(None)
+                return Some((n, tokens));
             }
-        } else {
-            Ok(None)
         }
+
+        None
     }
 }
 
@@ -840,7 +852,7 @@ fn test_ones_literal() {
 #[test]
 fn test_simple_num() {
     let lexemes = vec![Lexeme::Num(5)];
-    let (num, t) = Num::parse(lexemes.as_slice()).unwrap().unwrap();
+    let (num, t) = Num::parse(lexemes.as_slice()).unwrap();
 
     assert_eq!(num, 5);
     assert_eq!(t, 1);
@@ -856,7 +868,7 @@ fn test_complex_triple_num() {
         Lexeme::Dash,
         Lexeme::Five
     ];
-    let (num, t) = NumTriple::parse(lexemes.as_slice()).unwrap().unwrap();
+    let (num, t) = NumTriple::parse(lexemes.as_slice()).unwrap();
 
     assert_eq!(num, 235);
     assert_eq!(t, 6);
@@ -874,14 +886,16 @@ fn test_complex_num() {
         Lexeme::And,
         Lexeme::Ten
     ];
-    let (num, t) = Num::parse(lexemes.as_slice()).unwrap().unwrap();
+    let (num, t) = Num::parse(lexemes.as_slice()).unwrap();
 
+    assert_eq!(t, 8);
     assert_eq!(num, 205_030_010);
-    assert_eq!(t, 8)
 }
 
 #[test]
 fn test_simple_date_time() {
+    use chrono::Timelike;
+
     let lexemes = vec![
         Lexeme::February,
         Lexeme::Num(16),
@@ -891,8 +905,8 @@ fn test_simple_date_time() {
         Lexeme::Num(27),
         Lexeme::PM
     ];
-    let (date,t) = DateTime::parse(lexemes.as_slice()).unwrap().unwrap();
-    let date = date.to_chrono();
+    let (date,t) = DateTime::parse(lexemes.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
 
     assert_eq!(t, 7);
     assert_eq!(date.year(), 2022);
@@ -914,13 +928,17 @@ fn test_complex_relative_datetime() {
         Lexeme::The,
         Lexeme::Day,
         Lexeme::After,
-        Lexeme::Tomorrow
+        Lexeme::Tomorrow,
+        Lexeme::Comma,
+        Lexeme::Num(5),
+        Lexeme::Colon,
+        Lexeme::Num(20)
     ];
     let today = Local::now().naive_local().date();
-    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap().unwrap();
-    let date = date.to_chrono();
+    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
 
-    assert_eq!(t, 10);
+    assert_eq!(t, 14);
     assert_eq!(date.year(), today.year());
     assert_eq!(date.month(), today.month());
     assert_eq!(date.day(), today.day() + 7 - 2 + 1 + 1);
@@ -928,9 +946,11 @@ fn test_complex_relative_datetime() {
 
 #[test]
 fn test_datetime_now() {
+    use chrono::Timelike;
+
     let lexemes = vec![Lexeme::Now];
-    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap().unwrap();
-    let date = date.to_chrono();
+    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
 
     let now = Local::now().naive_local();
     assert_eq!(t, 1);
@@ -944,20 +964,20 @@ fn test_datetime_now() {
 #[test]
 fn test_malformed_article_after() {
     let lexemes = vec![Lexeme::A, Lexeme::Day, Lexeme::After, Lexeme::Colon];
-    assert!(DateTime::parse(lexemes.as_slice()).is_err());
+    assert!(DateTime::parse(lexemes.as_slice()).is_none());
 }
 
 #[test]
 fn test_malformed_after() {
     let lexemes = vec![Lexeme::Num(5), Lexeme::Day, Lexeme::After, Lexeme::Colon];
-    assert!(DateTime::parse(lexemes.as_slice()).is_err());
+    assert!(DateTime::parse(lexemes.as_slice()).is_none());
 }
 
 #[test]
 fn test_datetime_ago() {
-    let lexemes = vec![Lexeme::One, Lexeme::Day, Lexeme::Ago];
-    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap().unwrap();
-    let date = date.to_chrono();
+    let lexemes = vec![Lexeme::A, Lexeme::Day, Lexeme::Ago];
+    let (date, t) = DateTime::parse(lexemes.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
 
     let today = Local::now().naive_local().date();
     assert_eq!(t, 3);
@@ -982,11 +1002,135 @@ fn test_teens() {
 
 #[test]
 fn test_article_before() {
-    let (date, t) = DateTime::parse(&[Lexeme::A, Lexeme::Day, Lexeme::Before, Lexeme::Today]).unwrap().unwrap();
-    let date = date.to_chrono();
+    let (date, t) = DateTime::parse(&[Lexeme::A, Lexeme::Day, Lexeme::Before, Lexeme::Today]).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
     let today = Local::now().naive_local().date();
     assert_eq!(t, 4);
     assert_eq!(date.year(), today.year());
     assert_eq!(date.month(), today.month());
     assert_eq!(date.day(), today.day()-1);
 }
+
+#[test]
+fn test_after_december() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Month,
+        Lexeme::After,
+        Lexeme::December,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year() + 1);
+    assert_eq!(date.month(), 1);
+    assert_eq!(date.day(), 5);
+}
+
+#[test]
+fn test_month_before_january() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Month,
+        Lexeme::Before,
+        Lexeme::January,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year() - 1);
+    assert_eq!(date.month(), 12);
+    assert_eq!(date.day(), 5);
+
+}
+
+#[test]
+fn test_month_after() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Month,
+        Lexeme::After,
+        Lexeme::October,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year());
+    assert_eq!(date.month(), 11);
+    assert_eq!(date.day(), 5);
+}
+
+#[test]
+fn test_year_after() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Year,
+        Lexeme::After,
+        Lexeme::October,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year()+1);
+    assert_eq!(date.month(), 10);
+    assert_eq!(date.day(), 5);
+
+}
+
+#[test]
+fn test_month_before() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Month,
+        Lexeme::Before,
+        Lexeme::October,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year());
+    assert_eq!(date.month(), 9);
+    assert_eq!(date.day(), 5);
+
+}
+
+#[test]
+fn test_year_before() {
+    let l = vec![
+        Lexeme::A,
+        Lexeme::Year,
+        Lexeme::Before,
+        Lexeme::October,
+        Lexeme::Num(5)
+    ];
+
+    let today = Local::now().naive_local().date();
+    let (date, t) = DateTime::parse(l.as_slice()).unwrap();
+    let date = date.to_chrono(Local::now().naive_local().time());
+
+    assert_eq!(t, 5);
+    assert_eq!(date.year(), today.year()-1);
+    assert_eq!(date.month(), 10);
+    assert_eq!(date.day(), 5);
+
+}
