@@ -18,13 +18,13 @@
 //!
 //! fn main() {
 //!     let date_string = "Five days after 2/12/22 5:00 PM";
-//!     let date = parse(date_string);
+//!     let date = parse(date_string).unwrap();
 //!     println!("{:?}", date);
 //! }
 //! ```
 //!
 //! Any relevant date time information not specified is assumed to be
-//! the current information.
+//! the value of the current date time.
 //!
 //! ## Grammar
 //! ```text
@@ -32,14 +32,15 @@
 //!              | <date> <time>
 //!              | <date> , <time>
 //!              | <duration> after <datetime>
-//!              | a <unit> after <datetime>
-//!              | the <unit> after <datetime>
-//!              | a <unit> before <datetime>
-//!              | the <unit> before <datetime>
 //!              | <duration> from <datetime>
+//!              | <duration> before <datetime>
 //!              | <duration> ago
 //!              | now
 //!
+//! <article> ::= a
+//!            | an
+//!            | the
+//!
 //! <date> ::= today
 //!          | tomorrow
 //!          | yesterday
@@ -95,6 +96,7 @@
 //!           | dec
 //!
 //! <duration> ::= <num> <unit>
+//!              | <article> <unit>
 //!
 //! <time> ::= <num>:<num>
 //!          | <num>:<num> am
@@ -181,22 +183,31 @@
 mod lexer;
 mod ast;
 
-use chrono::{NaiveDateTime, Datelike};
+use chrono::{NaiveDateTime, NaiveTime, Local};
 
-/// Parse an input string into a chrono NaiveDateTime
-pub fn parse(input: &str) -> Result<NaiveDateTime, String> {
+/// Parse an input string into a chrono NaiveDateTime, using the default
+/// values from the specified default value where not specified
+pub fn parse_with_default_time(input: &str, default: NaiveTime) -> Result<NaiveDateTime, String> {
     let lexemes = lexer::Lexeme::lex_line(input.into())?;
-    let tree = ast::DateTime::parse(lexemes.as_slice())?;
+    let tree = ast::DateTime::parse(lexemes.as_slice());
 
     if tree.is_none() {
-        return Err("No DateTime Found".into());
+        return Err("Unrecognized Date Format".into());
     }
 
-    Ok(tree.unwrap().0.to_chrono())
+    let date = tree.unwrap().0.to_chrono(default);
+    Ok(date)
+}
+
+/// Parse an input string into a chrono NaiveDateTime with the default
+/// time being now
+pub fn parse(input: &str) -> Result<NaiveDateTime, String> {
+    parse_with_default_time(input, Local::now().naive_local().time())
 }
 
 #[test]
 fn test_parse() {
+    use chrono::Datelike;
     let input = "2/12/2022";
     let date = parse(input).unwrap();
 
